package org.eclipse.nebula.widgets.xviewer.customize.dialog;

import java.util.ArrayList;
import java.util.List;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.nebula.widgets.xviewer.XViewer;
import org.eclipse.nebula.widgets.xviewer.XViewerColumn;
import org.eclipse.nebula.widgets.xviewer.XViewerFactory;
import org.eclipse.nebula.widgets.xviewer.XViewerLabelProvider;
import org.eclipse.nebula.widgets.xviewer.customize.ColumnFilterData;
import org.eclipse.nebula.widgets.xviewer.customize.CustomizeData;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TreeItem;

/**
 * Customization dialog UI section for configuring column filters.
 * *
 *
 * @author Chris Merrill
 */
public class ColumnFilterCustomUISection implements CustomizeDialogUISection {
   private XViewer viewer;
   private List<FilterData> filters = new ArrayList<FilterData>();
   public final static XViewerColumn COLUMN_NAME = new XViewerColumn("Name", "Name", 200, SWT.LEFT, true, XViewerColumn.SortDataType.String, true, "The name of the field to filter");
   public final static XViewerColumn COLUMN_FILTER_VALUE = new XViewerColumn("Filter_Value", "Filter Value", 200, SWT.LEFT, true, XViewerColumn.SortDataType.String, true, "The value to filter by (prefix with ! to invert the match)");
   public final static XViewerColumn[] COLUMNS = {COLUMN_NAME, COLUMN_FILTER_VALUE};

   @Override
   public Control createSection(Composite parent) {
      viewer = new XViewer(parent, SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL | SWT.FULL_SELECTION, new ColumnFilterUIViewerFactory());
      viewer.getTree().setItemCount(5);
      viewer.setContentProvider(new ColumnFilterUIContentProvider());
      viewer.setCellEditors(new CellEditor[]{
            new TextCellEditor(viewer.getTree()),
            new TextCellEditor(viewer.getTree()),
      });
      viewer.setColumnProperties(new String[] {COLUMN_NAME.getId(), COLUMN_FILTER_VALUE.getId()});
      viewer.setCellModifier(new ICellModifier() {
         public boolean canModify(Object element, String property) {
            FilterData data = (FilterData) element;
            if (!data.isVisible())
               return false;
            if (property.equals(COLUMN_FILTER_VALUE.getId()))
               return true;
            return false;
         }

         public Object getValue(Object element, String property) {
            if (property.equals(COLUMN_FILTER_VALUE.getId())) {
               FilterData data = (FilterData) element;
               return data.getFilterText();
            }
            return null;
         }

         public void modify(Object element, String property, Object value) {
            if (property.equals(COLUMN_FILTER_VALUE.getId())) {
               TreeItem item = (TreeItem) element;
               FilterData data = (FilterData) item.getData();
               data.setFilterText((String) value);
               viewer.update(data, new String[] {COLUMN_FILTER_VALUE.getId()});
            }
         }
      });

      viewer.setLabelProvider(new ColumnFilterUILabelProvider(viewer));
      return viewer.getTree();
   }

   @Override
   public String getName() {
      return "Column Filters";
   }

   @Override
   public void readFromCustomData(CustomizeData customData) {
      ColumnFilterData filter_data = customData.getColumnFilterData();

      filters.clear();
      for (XViewerColumn column : customData.getColumnData().getColumns()) {
         String id = column.getId();
         String filter_text = filter_data.getFilterText(id);
         if (filter_text == null)
            filter_text = "";
         FilterData data = new FilterData(id, customData.getColumnData().getXColumn(id).getName(), filter_text, customData.getColumnData().getXColumn(id).isShow());
         filters.add(data);
      }

      viewer.setInput(filters);
   }

   @Override
   public void saveToCustomData(CustomizeData custom_data) {
      for (FilterData data : filters) {
         String filter_text = data.getFilterText();
         if (filter_text.length() == 0)
            custom_data.getColumnFilterData().removeFilterText(data.getId());
         else
            custom_data.getColumnFilterData().setFilterText(data.getId(), filter_text);
      }
   }

   @Override
   public void columnVisibilityChanged(java.util.List<XViewerColumn> visible, java.util.List<XViewerColumn> hidden) {
      for (XViewerColumn column : visible) {
         filters.get(filters.indexOf(new FilterData(column.getId(), null, null, true))).setVisible(true);
      }
      for (XViewerColumn column : hidden) {
         FilterData filterData = filters.get(filters.indexOf(new FilterData(column.getId(), null, null, true)));
         filterData.setFilterText("");
         filterData.setVisible(false);
      }

      viewer.setInput(filters);
   }
}

class ColumnFilterUIViewerFactory extends XViewerFactory {
   ColumnFilterUIViewerFactory() {
      super("column filters");
      registerColumns(ColumnFilterCustomUISection.COLUMNS);
   }

   @Override
   public boolean isAdmin() {
      return false;
   }

   @Override
   public boolean isFilterUiAvailable() {
      return false;
   }

   @Override
   public boolean isHeaderBarAvailable() {
      return false;
   }

   @Override
   public boolean isLoadedStatusLabelAvailable() {
      return false;
   }

   @Override
   public boolean isSearchUiAvailable() {
      return false;
   }
}

class ColumnFilterUIContentProvider implements ITreeContentProvider {
   private List<FilterData> input;

   @Override
   public Object[] getElements(Object inputElement) {
      List<FilterData> visible_fields = new ArrayList<FilterData>();
      for (FilterData data : input)
         if (data.isVisible())
            visible_fields.add(data);
      return visible_fields.toArray();
   }

   @Override
   public Object[] getChildren(Object parentElement) {
      return new Object[0];
   }

   @Override
   public Object getParent(Object element) {
      return null;
   }

   @Override
   public boolean hasChildren(Object element) {
      return false;
   }

   @Override
   public void dispose() {
   }

   @Override
   public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
      input = (List<FilterData>) newInput;
   }
}

class ColumnFilterUILabelProvider extends XViewerLabelProvider {
   ColumnFilterUILabelProvider(XViewer viewer) {
      super(viewer);
   }

   @Override
   public Image getColumnImage(Object element, XViewerColumn xCol, int columnIndex) throws Exception {
      return null;
   }

   @Override
   public String getColumnText(Object element, XViewerColumn xCol, int columnIndex) throws Exception {
      FilterData data = (FilterData) element;
      if (xCol.equals(ColumnFilterCustomUISection.COLUMN_NAME))
         return data.getName();
      else if (xCol.equals(ColumnFilterCustomUISection.COLUMN_FILTER_VALUE))
         return data.getFilterText();
      return null;
   }

   @Override
   public void addListener(ILabelProviderListener listener) {
   }

   @Override
   public void dispose() {
   }

   @Override
   public boolean isLabelProperty(Object element, String property) {
      if (property.equals(ColumnFilterCustomUISection.COLUMN_FILTER_VALUE.getId()))
         return true;
      return false;
   }

   @Override
   public void removeListener(ILabelProviderListener listener) {
   }
}

class FilterData {
   private String id;
   private String name;
   private String filterText;
   private boolean visible;

   FilterData(String id, String name, String filterText, boolean visible) {
      this.id = id;
      this.name = name;
      this.filterText = filterText;
      this.visible = visible;
   }

   public String getId() {
      return id;
   }

   public String getName() {
      return name;
   }

   public String getFilterText() {
      return filterText;
   }

   public void setFilterText(String filter) {
      this.filterText = filter;
   }

   public boolean isVisible() {
      return visible;
   }

   public void setVisible(boolean visible) {
      this.visible = visible;
   }

   @Override
   public boolean equals(Object obj) {
      if (!(obj instanceof FilterData))
         return false;
      return ((FilterData)obj).id.equals(id);
   }
}
