package org.eclipse.nebula.widgets.xviewer.customize.dialog;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.nebula.widgets.xviewer.Activator;
import org.eclipse.nebula.widgets.xviewer.XViewerColumn;
import org.eclipse.nebula.widgets.xviewer.XViewerColumnLabelProvider;
import org.eclipse.nebula.widgets.xviewer.XViewerColumnSorter;
import org.eclipse.nebula.widgets.xviewer.customize.CustomizeData;
import org.eclipse.nebula.widgets.xviewer.customize.CustomizeManager;
import org.eclipse.nebula.widgets.xviewer.util.internal.ArrayTreeContentProvider;
import org.eclipse.nebula.widgets.xviewer.util.internal.CollectionsUtil;
import org.eclipse.nebula.widgets.xviewer.util.internal.XViewerFilteredTree;
import org.eclipse.nebula.widgets.xviewer.util.internal.XViewerLog;
import org.eclipse.nebula.widgets.xviewer.util.internal.dialog.DialogWithEntry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

/**
 * Customization dialog UI section for choosing which columns to show.
 *
 * @author Chris Merrill
 */
public class ColumnsCustomUISection implements CustomizeDialogUISection {
   private CustomizeManager customizeManager;
   private XViewerCustomizeDialog dialog;
   private XViewerFilteredTree hiddenColTable;
   private XViewerFilteredTree visibleColTable;
   private Button addItemButton;
   private Button removeItemButton;
   private Button moveUpButton;
   private Button moveDownButton;
   private Button renameButton;
   private boolean isFeedbackAfter = false;


   public void initialize(CustomizeManager customizeManager, XViewerCustomizeDialog dialog) {
      this.customizeManager = customizeManager;
      this.dialog = dialog;
   }

   @Override
   public String getName() {
      return "Columns";
   }

   @Override
   public Control createSection(Composite parent) {
      Composite body = new Composite(parent, SWT.NONE);
      final GridLayout layout = new GridLayout();
      layout.numColumns = 3;
      body.setLayout(layout);

      createHiddenSection(body);
      createMiddleSection(body);
      createVisibleSection(body);

      renameButton = new Button(body, SWT.NONE);
      renameButton.setText("Rename Column");
      renameButton.addSelectionListener(new SelectionAdapter() {
         @Override
         public void widgetSelected(SelectionEvent e) {
            super.widgetSelected(e);
            handleRenameButton();
         }
      });

      updateButtonEnablements();
      return body;
   }

   private void createHiddenSection(Composite parent) {
      Composite hiddenTableComp = new Composite(parent, SWT.NONE);
      hiddenTableComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
      GridLayout gridLayout_4 = new GridLayout();
      gridLayout_4.marginWidth = 0;
      gridLayout_4.marginHeight = 0;
      hiddenTableComp.setLayout(gridLayout_4);

      Label hiddenColumnsLabel = new Label(hiddenTableComp, SWT.NONE);
      hiddenColumnsLabel.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
      hiddenColumnsLabel.setText("Hidden Columns");

      // Hidden Column Table
      hiddenColTable = new XViewerFilteredTree(hiddenTableComp);
      Tree table_1 = hiddenColTable.getViewer().getTree();
      GridData gd_table_1 = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 2);
      gd_table_1.widthHint = 300;
      table_1.setLayoutData(gd_table_1);
      hiddenColTable.getViewer().setLabelProvider(new XViewerColumnLabelProvider());
      hiddenColTable.getViewer().setContentProvider(new ArrayTreeContentProvider());
      hiddenColTable.getViewer().setSorter(new XViewerColumnSorter());
      hiddenColTable.getViewer().addSelectionChangedListener(new ISelectionChangedListener() {
         @Override
         public void selectionChanged(SelectionChangedEvent event) {
            updateButtonEnablements();
         }
      });

      hiddenColTable.getViewer().addDragSupport(DND.DROP_MOVE, new Transfer[]{TextTransfer.getInstance()}, hiddenTableDragListener);
      hiddenColTable.getViewer().addDropSupport(DND.DROP_MOVE, new Transfer[]{TextTransfer.getInstance()}, hiddenTableDropListener);
   }

   private void createMiddleSection(Composite parent) {
      Composite moveButtonComp = new Composite(parent, SWT.NONE);
      moveButtonComp.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
      GridLayout gridLayout_5 = new GridLayout();
      gridLayout_5.marginWidth = 0;
      gridLayout_5.marginHeight = 0;
      moveButtonComp.setLayout(gridLayout_5);

      addItemButton = new Button(moveButtonComp, SWT.NONE);
      addItemButton.setText(">");
      addItemButton.setToolTipText("Add");
      addItemButton.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
      addItemButton.addSelectionListener(new SelectionAdapter() {
         @Override
         public void widgetSelected(SelectionEvent e) {
            super.widgetSelected(e);
            handleAddItemButton();
         }
      });

      Button addAllItemButton = new Button(moveButtonComp, SWT.NONE);
      addAllItemButton.setText(">>");
      addAllItemButton.setToolTipText("Add All");
      addAllItemButton.addSelectionListener(new SelectionAdapter() {
         @Override
         public void widgetSelected(SelectionEvent e) {
            super.widgetSelected(e);
            handleAddAllItemButton();
         }
      });

      removeItemButton = new Button(moveButtonComp, SWT.NONE);
      removeItemButton.setText("<");
      removeItemButton.setToolTipText("Remove");
      removeItemButton.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
      removeItemButton.addSelectionListener(new SelectionAdapter() {
         @Override
         public void widgetSelected(SelectionEvent e) {
            super.widgetSelected(e);
            handleRemoveItemButton();
         }
      });

      Button removeAllItemButton = new Button(moveButtonComp, SWT.NONE);
      removeAllItemButton.setText("<<");
      removeAllItemButton.setToolTipText("Remove All");
      removeAllItemButton.addSelectionListener(new SelectionAdapter() {
         @Override
         public void widgetSelected(SelectionEvent e) {
            super.widgetSelected(e);
            handleRemoveAllItemButton();
         }
      });

      moveUpButton = new Button(moveButtonComp, SWT.NONE);
      moveUpButton.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
      moveUpButton.setText("^");
      moveUpButton.setToolTipText("Move Up");
      moveUpButton.addSelectionListener(new SelectionAdapter() {
         @Override
         public void widgetSelected(SelectionEvent e) {
            super.widgetSelected(e);
            handleMoveUpButton();
         }
      });

      moveDownButton = new Button(moveButtonComp, SWT.NONE);
      moveDownButton.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
      moveDownButton.setText("v");
      moveDownButton.setToolTipText("Move Down");
      moveDownButton.addSelectionListener(new SelectionAdapter() {
         @Override
         public void widgetSelected(SelectionEvent e) {
            super.widgetSelected(e);
            handleMoveDownButton();
         }
      });

   }

   private void createVisibleSection(Composite parent) {
      final Composite visibleTableComp = new Composite(parent, SWT.NONE);
      visibleTableComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
      final GridLayout gridLayout_6 = new GridLayout();
      gridLayout_6.marginWidth = 0;
      gridLayout_6.marginHeight = 0;
      visibleTableComp.setLayout(gridLayout_6);

      final Label visibleColumnsLabel = new Label(visibleTableComp, SWT.NONE);
      visibleColumnsLabel.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
      visibleColumnsLabel.setText("Visible Columns");

      // Visible Column Table
      visibleColTable = new XViewerFilteredTree(visibleTableComp);
      final Tree table = visibleColTable.getViewer().getTree();
      final GridData gd_table = new GridData(SWT.FILL, SWT.FILL, true, true);
      gd_table.widthHint = 300;
      table.setLayoutData(gd_table);
      visibleColTable.getViewer().setLabelProvider(new XViewerColumnLabelProvider());
      visibleColTable.getViewer().setContentProvider(new ArrayTreeContentProvider());
      visibleColTable.getViewer().addSelectionChangedListener(new ISelectionChangedListener() {
         @Override
         public void selectionChanged(SelectionChangedEvent event) {
            updateButtonEnablements();
         }
      });
      visibleColTable.getViewer().addDragSupport(DND.DROP_MOVE, new Transfer[]{TextTransfer.getInstance()}, visibleTableDragListener);
      visibleColTable.getViewer().addDropSupport(DND.DROP_MOVE, new Transfer[]{TextTransfer.getInstance()}, visibleTableDropListener);

   }

   public List<XViewerColumn> getVisibleColumns() {
      return (List<XViewerColumn>) visibleColTable.getViewer().getInput();
   }

   public List<XViewerColumn> getHiddenColumns() {
      return (List<XViewerColumn>) hiddenColTable.getViewer().getInput();
   }

   private List<XViewerColumn> getVisibleTableSelection() {
      return getTableSelection(visibleColTable.getViewer());
   }

   private List<XViewerColumn> getHiddenTableSelection() {
      return getTableSelection(hiddenColTable.getViewer());
   }

   private List<XViewerColumn> getTableSelection(TreeViewer xColTableViewer) {
      List<XViewerColumn> xCols = new ArrayList<XViewerColumn>();
      IStructuredSelection selection = (IStructuredSelection) xColTableViewer.getSelection();
      if (selection.isEmpty()) {
         return null;
      }
      Iterator<?> i = selection.iterator();
      while (i.hasNext()) {
         xCols.add((XViewerColumn) i.next());
      }
      return xCols;
   }

   private void updateButtonEnablements() {
      addItemButton.setEnabled(hiddenColTable.getViewer().getTree().isFocusControl() && getHiddenTableSelection() != null);
      removeItemButton.setEnabled(visibleColTable.getViewer().getTree().isFocusControl() && getVisibleTableSelection() != null);
      moveDownButton.setEnabled(visibleColTable.getViewer().getTree().isFocusControl() && getVisibleTableSelection() != null);
      moveUpButton.setEnabled(visibleColTable.getViewer().getTree().isFocusControl() && getVisibleTableSelection() != null);
      renameButton.setEnabled(visibleColTable.getViewer().getTree().isFocusControl() && getVisibleTableSelection() != null && getVisibleTableSelection().size() == 1);
   }


   DragSourceAdapter hiddenTableDragListener = new DragSourceAdapter() {
      @Override
      public void dragStart(DragSourceEvent event) {
         if (hiddenColTable.getViewer().getSelection().isEmpty()) {
            event.doit = false;
         }
      }

      /*
      * @see org.eclipse.swt.dnd.DragSourceAdapter#dragSetData(org.eclipse.swt.dnd.DragSourceEvent)
      */
      @Override
      public void dragSetData(DragSourceEvent event) {
         if (TextTransfer.getInstance().isSupportedType(event.dataType)) {
            java.util.List<XViewerColumn> selCols = getHiddenTableSelection();
            Collection<String> ids = new ArrayList<String>(selCols.size());

            for (XViewerColumn xCol : selCols) {
               ids.add(xCol.getId());
            }

            event.data = CollectionsUtil.toString(ids, null, ", ", null);
         }
      }
   };

   DropTargetAdapter hiddenTableDropListener = new DropTargetAdapter() {

      @Override
      public void dragOperationChanged(DropTargetEvent event) {
         // do nothing
      }

      @Override
      public void drop(DropTargetEvent event) {
         if (event.data instanceof String) {
            performHiddenTableTextDrop(event);
         }
      }

      @Override
      public void dragOver(DropTargetEvent event) {
         performHiddenTableDragOver(event);
      }

      @Override
      public void dropAccept(DropTargetEvent event) {
         // do nothing
      }
   };

   DragSourceAdapter visibleTableDragListener = new DragSourceAdapter() {
      @Override
      public void dragStart(DragSourceEvent event) {
         if (visibleColTable.getViewer().getSelection().isEmpty()) {
            event.doit = false;
         }
      }

      /*
      * @see org.eclipse.swt.dnd.DragSourceAdapter#dragSetData(org.eclipse.swt.dnd.DragSourceEvent)
      */
      @Override
      public void dragSetData(DragSourceEvent event) {
         if (TextTransfer.getInstance().isSupportedType(event.dataType)) {
            List<XViewerColumn> selCols = getVisibleTableSelection();
            Collection<String> ids = new ArrayList<String>(selCols.size());

            for (XViewerColumn xCol : selCols) {
               ids.add(xCol.getId());
            }

            event.data = CollectionsUtil.toString(ids, null, ", ", null);
         }
      }
   };
   DropTargetAdapter visibleTableDropListener = new DropTargetAdapter() {

      @Override
      public void dragOperationChanged(DropTargetEvent event) {
         // do nothing
      }

      @Override
      public void drop(DropTargetEvent event) {
         if (event.data instanceof String) {
            performVisibleTableTextDrop(event);
         }
      }

      @Override
      public void dragOver(DropTargetEvent event) {
         performVisibleTableDragOver(event);
      }

      @Override
      public void dropAccept(DropTargetEvent event) {
         // do nothing
      }
   };

   /**
    * Drag should only be from visible table
    */
   public void performHiddenTableDragOver(DropTargetEvent event) {
      if (!TextTransfer.getInstance().isSupportedType(event.currentDataType)) {
         event.detail = DND.DROP_NONE;
         return;
      }
      // Only allow drag from visibleColTable
      if (event.widget != visibleColTable) {
         return;
      }

      event.detail = DND.DROP_MOVE;
   }

   @SuppressWarnings("unchecked")
   public void performHiddenTableTextDrop(DropTargetEvent event) {

      String droppedIds = (String) event.data;

      List<XViewerColumn> droppedVisibleTableXCols = new ArrayList<XViewerColumn>();
      List<XViewerColumn> orderCols = (List<XViewerColumn>) visibleColTable.getViewer().getInput();
      for (XViewerColumn xCol : orderCols) {
         if (droppedIds.contains(xCol.getId())) {
            droppedVisibleTableXCols.add(xCol);
         }
      }

      moveFromVisibleToHidden(droppedVisibleTableXCols);
   }

   @SuppressWarnings("unchecked")
   public void performVisibleTableTextDrop(DropTargetEvent event) {
      Tree tree = visibleColTable.getViewer().getTree();
      TreeItem dragOverTreeItem = tree.getItem(visibleColTable.getViewer().getTree().toControl(event.x, event.y));

      String droppedIds = (String) event.data;

      // Determine dragOverXCol, if any
      XViewerColumn dragOverXCol = null;
      if (dragOverTreeItem != null) {
         dragOverXCol = (XViewerColumn) dragOverTreeItem.getData();
         // Don't allow dropping on same item as dragging
         if (droppedIds.contains(dragOverXCol.getId())) {
            return;
         }
      }

      List<XViewerColumn> droppedXCols = new ArrayList<XViewerColumn>();
      List<XViewerColumn> orderCols = (List<XViewerColumn>) visibleColTable.getViewer().getInput();
      for (XViewerColumn xCol : orderCols) {
         if (droppedIds.contains(xCol.getId())) {
            droppedXCols.add(xCol);
         }
      }
      for (XViewerColumn xCol : (List<XViewerColumn>) hiddenColTable.getViewer().getInput()) {
         if (droppedIds.contains(xCol.getId())) {
            droppedXCols.add(xCol);
         }
      }
      orderCols.removeAll(droppedXCols);

      int dropXColOrderColsIndex = 0;
      for (XViewerColumn xCol : (List<XViewerColumn>) visibleColTable.getViewer().getInput()) {
         if (dragOverXCol != null && xCol.getId().equals(dragOverXCol.getId())) {
            break;
         }
         dropXColOrderColsIndex++;
      }

      if (isFeedbackAfter) {
         orderCols.addAll(dropXColOrderColsIndex + 1, droppedXCols);
      }
      else {
         orderCols.addAll(dropXColOrderColsIndex, droppedXCols);
      }
      visibleColTable.getViewer().setInput(orderCols);

      List<XViewerColumn> hiddenCols = (List<XViewerColumn>) hiddenColTable.getViewer().getInput();
      hiddenCols.removeAll(droppedXCols);
      hiddenColTable.getViewer().setInput(hiddenCols);
   }

   public void performVisibleTableDragOver(DropTargetEvent event) {
      if (!TextTransfer.getInstance().isSupportedType(event.currentDataType)) {
         event.detail = DND.DROP_NONE;
         return;
      }

      Tree tree = visibleColTable.getViewer().getTree();
      TreeItem dragOverTreeItem = tree.getItem(visibleColTable.getViewer().getTree().toControl(event.x, event.y));
      if (dragOverTreeItem == null) {
         return;
      }

      event.feedback = DND.FEEDBACK_EXPAND;
      event.detail = DND.DROP_NONE;

      IStructuredSelection selectedItem = (IStructuredSelection) visibleColTable.getViewer().getSelection();
      if (selectedItem == null || selectedItem.isEmpty()) {
         selectedItem = (IStructuredSelection) hiddenColTable.getViewer().getSelection();
      }
      if (selectedItem == null) {
         return;
      }
      Object obj = selectedItem.getFirstElement();
      if (obj instanceof XViewerColumn) {
         if (isFeedbackAfter) {
            event.feedback = DND.FEEDBACK_INSERT_AFTER;
         }
         else {
            event.feedback = DND.FEEDBACK_INSERT_BEFORE;
         }
         event.detail = DND.DROP_MOVE;
      }
   }

   @SuppressWarnings("unchecked")
   private void moveFromVisibleToHidden(List<XViewerColumn> visibleSelCols) {
      // Remove from visible
      if (visibleSelCols == null || visibleSelCols.isEmpty()) {
         return;
      }
      List<XViewerColumn> visibleCols = (List<XViewerColumn>) visibleColTable.getViewer().getInput();
      visibleCols.removeAll(visibleSelCols);
      visibleColTable.getViewer().setInput(visibleCols);

      // Add to hidden
      List<XViewerColumn> hiddenCols = (List<XViewerColumn>) hiddenColTable.getViewer().getInput();
      hiddenCols.addAll(visibleSelCols);
      hiddenColTable.getViewer().setInput(hiddenCols);

      dialog.columnVisibilityChanged(visibleCols, hiddenCols);
   }

   @SuppressWarnings("unchecked")
   private void handleAddItemButton() {
      // Remove from hidden
      List<XViewerColumn> hiddenSelCols = getHiddenTableSelection();
      if (hiddenSelCols == null) {
         return;
      }
      List<XViewerColumn> hiddenCols = (List<XViewerColumn>) hiddenColTable.getViewer().getInput();
      hiddenCols.removeAll(hiddenSelCols);
      hiddenColTable.getViewer().setInput(hiddenCols);

      // Add to visible
      List<XViewerColumn> visibleCols = (List<XViewerColumn>) visibleColTable.getViewer().getInput();
      visibleCols.addAll(hiddenSelCols);
      visibleColTable.getViewer().setInput(visibleCols);

      dialog.columnVisibilityChanged(visibleCols, hiddenCols);
   }

   private void handleRemoveItemButton() {
      List<XViewerColumn> visibleSelCols = getVisibleTableSelection();
      if (visibleSelCols != null && visibleSelCols.size() != 0) {
         moveFromVisibleToHidden(visibleSelCols);
      }
   }

   @SuppressWarnings("unchecked")
   private void handleAddAllItemButton() {

      List<XViewerColumn> hiddenCols = (List<XViewerColumn>) hiddenColTable.getViewer().getInput();

      List<XViewerColumn> visibleCols = (List<XViewerColumn>) visibleColTable.getViewer().getInput();

      visibleCols.addAll(hiddenCols);
      visibleColTable.getViewer().setInput(visibleCols);

      hiddenCols.clear();
      hiddenColTable.getViewer().setInput(hiddenCols);

      dialog.columnVisibilityChanged(visibleCols, hiddenCols);
   }

   /**
    * for testing purposes
    */
   protected void handleAddAllItem() {
      handleAddAllItemButton();
   }

   @SuppressWarnings("unchecked")
   private void handleRemoveAllItemButton() {

      List<XViewerColumn> visibleCols = (List<XViewerColumn>) visibleColTable.getViewer().getInput();

      List<XViewerColumn> hiddenCols = (List<XViewerColumn>) hiddenColTable.getViewer().getInput();
      hiddenCols.addAll(visibleCols);
      hiddenColTable.getViewer().setInput(hiddenCols);

      // Add to visible
      visibleCols.clear();
      visibleColTable.getViewer().setInput(visibleCols);

      dialog.columnVisibilityChanged(visibleCols, hiddenCols);
   }

   @SuppressWarnings("unchecked")
   private void handleMoveUpButton() {
      List<XViewerColumn> selCols = getVisibleTableSelection();
      if (selCols == null) {
         return;
      }
      List<XViewerColumn> orderCols = (List<XViewerColumn>) visibleColTable.getViewer().getInput();
      int index = orderCols.indexOf(selCols.iterator().next());
      if (index > 0) {
         orderCols.removeAll(selCols);
         orderCols.addAll(index - 1, selCols);
         visibleColTable.getViewer().setInput(orderCols);
      }
      else {
         return;
      }
      ArrayList<XViewerColumn> selected = new ArrayList<XViewerColumn>();
      selected.addAll(selCols);
      visibleColTable.getViewer().setSelection(new StructuredSelection(selected.toArray(new Object[selected.size()])));
      visibleColTable.getViewer().getTree().setFocus();
      updateButtonEnablements();
   }

   @SuppressWarnings("unchecked")
   private void handleMoveDownButton() {
      List<XViewerColumn> selCols = getVisibleTableSelection();
      if (selCols == null) {
         return;
      }
      List<XViewerColumn> orderCols = (List<XViewerColumn>) visibleColTable.getViewer().getInput();
      int index = orderCols.indexOf(selCols.iterator().next());
      if (index < (orderCols.size() - selCols.size())) {
         orderCols.removeAll(selCols);
         orderCols.addAll(index + 1, selCols);
         visibleColTable.getViewer().setInput(orderCols);
      }
      else {
         return;
      }
      ArrayList<XViewerColumn> selected = new ArrayList<XViewerColumn>();
      selected.addAll(selCols);
      visibleColTable.getViewer().setSelection(new StructuredSelection(selected.toArray(new Object[selected.size()])));
      visibleColTable.getViewer().getTree().setFocus();
      updateButtonEnablements();
   }

   private void handleRenameButton() {
      XViewerColumn xCol = getVisibleTableSelection().iterator().next();
      DialogWithEntry ed =
            new DialogWithEntry(Display.getCurrent().getActiveShell(), "Rename Column", null, "Enter new name",
                  MessageDialog.QUESTION, new String[]{"OK", "Use Default", "Cancel"}, 0);
      int result = ed.open();
      if (result == 2) {
         return;
      }
      if (result == 0) {
         customizeManager.customizeColumnName(xCol, ed.getEntry());
      }
      else if (result == 1) {
         customizeManager.customizeColumnName(xCol, "");
      }
      visibleColTable.getViewer().update(xCol, null);
   }


   @Override
   public void readFromCustomData(CustomizeData customData) {
      if (customData == null) {
         XViewerLog.logAndPopup(Activator.class, Level.SEVERE, new IllegalStateException("Can't obtain selection Xml"));
         return;
      }

      List<XViewerColumn> hideXCols = new ArrayList<XViewerColumn>();
      List<XViewerColumn> showXCols = new ArrayList<XViewerColumn>();
      for (XViewerColumn xCol : customData.getColumnData().getColumns()) {
         if (xCol.isShow()) {
            showXCols.add(xCol);
         }
         else {
            hideXCols.add(xCol);
         }
      }

      hiddenColTable.getViewer().setInput(hideXCols);
      visibleColTable.getViewer().setInput(showXCols);
   }

   @Override
   public void saveToCustomData(CustomizeData customData) {
      List<XViewerColumn> xCols = new ArrayList<XViewerColumn>();
      for (XViewerColumn xCol : (List<XViewerColumn>) visibleColTable.getViewer().getInput()) {
         xCol.setShow(true);
         xCols.add(xCol);
      }
      for (XViewerColumn xCol : (List<XViewerColumn>) hiddenColTable.getViewer().getInput()) {
         xCol.setShow(false);
         xCols.add(xCol);
      }
      customData.getColumnData().setColumns(xCols);
   }

   @Override
   public void columnVisibilityChanged(List<XViewerColumn> visible, List<XViewerColumn> hidden) {
      // This section is the source of these events, so no need to take action.
   }
}


