package org.eclipse.nebula.widgets.xviewer.customize.dialog;

import java.util.ArrayList;
import java.util.List;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxCellEditor;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.nebula.widgets.xviewer.XViewer;
import org.eclipse.nebula.widgets.xviewer.XViewerColumn;
import org.eclipse.nebula.widgets.xviewer.XViewerFactory;
import org.eclipse.nebula.widgets.xviewer.XViewerLabelProvider;
import org.eclipse.nebula.widgets.xviewer.customize.CustomizeData;
import org.eclipse.nebula.widgets.xviewer.customize.SortingData;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TreeItem;

/**
 * Customization dialog UI section for configuring column sort settings.
 *
 * @author Chris Merrill
 */
public class SortCustomUISection implements CustomizeDialogUISection {
   private XViewer viewer;
   private List<SortData> sorts = new ArrayList<SortData>();
   public final static XViewerColumn COLUMN_NAME = new XViewerColumn("NAME", "Name", 200, SWT.LEFT, true, XViewerColumn.SortDataType.String, true, "The name of the field to sort");
   public final static XViewerColumn COLUMN_SORT = new XViewerColumn("SORT", "Sort", 200, SWT.LEFT, true, XViewerColumn.SortDataType.String, true, "Sort? Y/N");
   public final static XViewerColumn COLUMN_ORDER = new XViewerColumn("ORDER", "Order", 200, SWT.LEFT, true, XViewerColumn.SortDataType.String, true, "Ascending? Y/N");
   public final static XViewerColumn[] COLUMNS = {COLUMN_NAME, COLUMN_SORT, COLUMN_ORDER};

   @Override
   public Control createSection(Composite parent) {
      viewer = new XViewer(parent, SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL | SWT.FULL_SELECTION, new SortUIViewerFactory());
      viewer.getTree().setItemCount(5);
      viewer.setContentProvider(new SortUIContentProvider());
      viewer.setCellEditors(new CellEditor[]{
            new TextCellEditor(viewer.getTree()),
            new CheckboxCellEditor(viewer.getTree()),
            new CheckboxCellEditor(viewer.getTree()),
      });
      viewer.setColumnProperties(new String[] {COLUMN_NAME.getId(), COLUMN_SORT.getId(), COLUMN_ORDER.getId()});
      viewer.setCellModifier(new ICellModifier() {
         public boolean canModify(Object element, String property) {
            SortData data = (SortData) element;
            if (!data.isVisible())
               return false;
            if (property.equals(COLUMN_SORT.getId())
                  || property.equals(COLUMN_ORDER.getId()))
               return true;
            return false;
         }

         public Object getValue(Object element, String property) {
            SortData data = (SortData) element;
            if (property.equals(COLUMN_SORT.getId()))
               return data.isSort();
            if (property.equals(COLUMN_ORDER.getId()))
               return data.isAscending();
            return null;
         }

         public void modify(Object element, String property, Object value) {
            TreeItem item = (TreeItem) element;
            SortData data = (SortData) item.getData();
            if (property.equals(COLUMN_SORT.getId())) {
               data.setSort((Boolean)value);
               viewer.update(data, new String[] {COLUMN_SORT.getId()});
            }
            else if (property.equals(COLUMN_ORDER.getId())) {
               data.setAscending((Boolean)value);
               viewer.update(data, new String[] {COLUMN_ORDER.getId()});
            }
         }
      });

      viewer.setLabelProvider(new SortUILabelProvider(viewer));
      return viewer.getTree();
   }

   @Override
   public String getName() {
      return "Sort";
   }

   @Override
   public void readFromCustomData(CustomizeData customData) {
      sorts.clear();
      SortingData sortData = customData.getSortingData();
      for (XViewerColumn column : customData.getColumnData().getColumns()) {
         String id = column.getId();
         boolean sort = sortData.getSortingIds().contains(id);
         boolean ascending = column.isSortForward();
         SortData data = new SortData(id, customData.getColumnData().getXColumn(id).getName(), sort, ascending, customData.getColumnData().getXColumn(id).isShow());
         sorts.add(data);
      }

      viewer.setInput(sorts);
   }

   @Override
   public void saveToCustomData(CustomizeData customData) {
      SortingData sortingData = customData.getSortingData();
      for (SortData data : sorts) {
         XViewerColumn column = customData.getColumnData().getXColumn(data.getId());
         if (data.isSort())
            sortingData.addSortingName(data.getId());
         else
            sortingData.removeSortingName(data.getId());
         column.setSortForward(data.isAscending());
      }
   }

   @Override
   public void columnVisibilityChanged(java.util.List<XViewerColumn> visible, java.util.List<XViewerColumn> hidden) {
      for (XViewerColumn column : visible) {
         sorts.get(sorts.indexOf(new SortData(column.getId(), null, false, false, true))).setVisible(true);
      }
      for (XViewerColumn column : hidden) {
         SortData sortData = sorts.get(sorts.indexOf(new SortData(column.getId(), null, false, false, true)));
         sortData.setSort(false);
         sortData.setAscending(true);
         sortData.setVisible(false);
      }

      viewer.setInput(sorts);
   }
}

class SortData {
   private String id;
   private String name;
   private boolean sort;
   private boolean ascending;
   private boolean visible;

   SortData(String id, String name, boolean sort, boolean ascending, boolean visible) {
      this.id = id;
      this.name = name;
      this.sort = sort;
      this.ascending = ascending;
      this.visible = visible;
   }

   public String getId() {
      return id;
   }

   public String getName() {
      return name;
   }

   public boolean isSort() {
      return sort;
   }

   public void setSort(boolean sort) {
      this.sort = sort;
   }

   public boolean isAscending() {
      return ascending;
   }

   public void setAscending(boolean ascending) {
      this.ascending = ascending;
   }

   public boolean isVisible() {
      return visible;
   }

   public void setVisible(boolean visible) {
      this.visible = visible;
   }

   @Override
   public boolean equals(Object obj) {
      if (!(obj instanceof SortData))
         return false;
      return ((SortData)obj).id.equals(id);
   }
}

class SortUIContentProvider implements ITreeContentProvider {
   private List<SortData> input;

   @Override
   public Object[] getElements(Object inputElement) {
      List<SortData> visible_fields = new ArrayList<SortData>();
      for (SortData data : input)
         if (data.isVisible())
            visible_fields.add(data);
      return visible_fields.toArray();
   }

   @Override
   public Object[] getChildren(Object parentElement) {
      return new Object[0];
   }

   @Override
   public Object getParent(Object element) {
      return null;
   }

   @Override
   public boolean hasChildren(Object element) {
      return false;
   }

   @Override
   public void dispose() {
   }

   @Override
   public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
      input = (List<SortData>) newInput;
   }
}

class SortUILabelProvider extends XViewerLabelProvider {
   SortUILabelProvider(XViewer viewer) {
      super(viewer);
   }

   @Override
   public Image getColumnImage(Object element, XViewerColumn xCol, int columnIndex) throws Exception {
      return null;
   }

   @Override
   public String getColumnText(Object element, XViewerColumn xCol, int columnIndex) throws Exception {
      SortData data = (SortData) element;
      if (xCol.equals(SortCustomUISection.COLUMN_NAME))
         return data.getName();
      else if (xCol.equals(SortCustomUISection.COLUMN_SORT)) {
         if (data.isSort())
            return "yes";
         return "no";
      }
      else if (xCol.equals(SortCustomUISection.COLUMN_ORDER)) {
         if (data.isSort()) {
            if (data.isAscending())
               return "ascending";
            return "descending";
         }
         else
            return "";
      }
      return null;
   }

   @Override
   public void addListener(ILabelProviderListener listener) {
   }

   @Override
   public void dispose() {
   }

   @Override
   public boolean isLabelProperty(Object element, String property) {
      if (property.equals(SortCustomUISection.COLUMN_SORT.getId())
            || property.equals(SortCustomUISection.COLUMN_ORDER.getId()))
         return true;
      return false;
   }

   @Override
   public void removeListener(ILabelProviderListener listener) {
   }
}

class SortUIViewerFactory extends XViewerFactory {
   SortUIViewerFactory() {
      super("column sorts");
      registerColumns(SortCustomUISection.COLUMNS);
   }

   @Override
   public boolean isAdmin() {
      return false;
   }

   @Override
   public boolean isFilterUiAvailable() {
      return false;
   }

   @Override
   public boolean isHeaderBarAvailable() {
      return false;
   }

   @Override
   public boolean isLoadedStatusLabelAvailable() {
      return false;
   }

   @Override
   public boolean isSearchUiAvailable() {
      return false;
   }
}
