/*******************************************************************************
 * Copyright (c) 2004, 2007 Boeing.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Boeing - initial API and implementation
 *******************************************************************************/
package org.eclipse.nebula.widgets.xviewer.customize.dialog;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Arrays;
import java.util.logging.Level;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.nebula.widgets.xviewer.Activator;
import org.eclipse.nebula.widgets.xviewer.XViewer;
import org.eclipse.nebula.widgets.xviewer.XViewerColumn;
import org.eclipse.nebula.widgets.xviewer.customize.CustomizeData;
import org.eclipse.nebula.widgets.xviewer.customize.CustomizeDataLabelProvider;
import org.eclipse.nebula.widgets.xviewer.customize.CustomizeManager;
import org.eclipse.nebula.widgets.xviewer.util.XViewerException;
import org.eclipse.nebula.widgets.xviewer.util.internal.ArrayTreeContentProvider;
import org.eclipse.nebula.widgets.xviewer.util.internal.PatternFilter;
import org.eclipse.nebula.widgets.xviewer.util.internal.XViewerFilteredTree;
import org.eclipse.nebula.widgets.xviewer.util.internal.XViewerLib;
import org.eclipse.nebula.widgets.xviewer.util.internal.XViewerLog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.events.IExpansionListener;
import org.eclipse.ui.forms.widgets.ExpandableComposite;

/**
 * Provides dialog for table customization
 *
 * @author Donald G. Dunne
 */
public class XViewerCustomizeDialog extends MessageDialog {
   private String title = "Customize Table";
   private static String buttons[] = new String[]{"Ok", "Apply", "Cancel"};
   private final XViewer xViewerToCustomize;
   private XViewerFilteredTree custTable;
   Button setDefaultButton, deleteButton;
   Button saveButton;
   private final static String SET_AS_DEFAULT = " Set as Default ";
   private final static String REMOVE_DEFAULT = "Remove Default";

   ColumnsCustomUISection columns_section;
   List<CustomizeDialogUISection> sections = new java.util.ArrayList<CustomizeDialogUISection>();

   public XViewerCustomizeDialog(XViewer xViewer) {
      this(xViewer, new CustomizeDialogUISection[] {
         new ColumnsCustomUISection(),
         new SortCustomUISection(),
         new FilterCustomUISection(),
         new ColumnFilterCustomUISection()
      });
   }

   public XViewerCustomizeDialog(XViewer xViewer, CustomizeDialogUISection[] sections) {
      this(xViewer, Display.getCurrent().getActiveShell(), sections);
   }

   private XViewerCustomizeDialog(XViewer xViewer, Shell parentShell, CustomizeDialogUISection[] sections) {
      super(parentShell, "", null, "", MessageDialog.NONE, buttons, 0);
      this.xViewerToCustomize = xViewer;
      setShellStyle(getShellStyle() | SWT.RESIZE);

      this.sections.addAll(Arrays.asList(sections));
   }

   public void setTitle(String title) {
      this.title = title;
   }


   @Override
   protected Control createDialogArea(Composite parent) {
      if (getShell() != null) {
         getShell().setText(title);
      }
      parent.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

      Composite comp = new Composite(parent, SWT.NONE);
      GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
      gd.heightHint = 700;
      comp.setLayoutData(gd);
      GridLayout gridLayout_2 = new GridLayout();
      gridLayout_2.numColumns = 2;
      comp.setLayout(gridLayout_2);

      Label namespaceLabel = new Label(comp, SWT.NONE);
      GridData gridData = new GridData(SWT.CENTER, SWT.CENTER, false, false);
      gridData.horizontalSpan = 2;
      namespaceLabel.setLayoutData(gridData);
      namespaceLabel.setText("Customization Namespace: " + xViewerToCustomize.getXViewerFactory().getNamespace());

      createSelectCustomizationSection(comp);

      rightSideScroller = new ScrolledComposite(comp, SWT.V_SCROLL | SWT.H_SCROLL);
      rightSideScroller.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

      rightSideComposite = new Composite(rightSideScroller, SWT.NONE);
      rightSideScroller.setContent(rightSideComposite);
      rightSideScroller.setMinWidth(300);
      rightSideScroller.setExpandHorizontal(true);
      rightSideScroller.setExpandVertical(true);

      // add listener to recompute min height of custom sections
      rightSideComposite.addListener(SWT.Resize, new Listener() {
         int width = -1;
         @Override
         public void handleEvent(Event event) {
            recomputeScrolledHeight();
         }
      });

      // Column Configuration
      GridLayout gridLayout = new GridLayout();
      gridLayout.marginWidth = 3;
      gridLayout.marginHeight = 3;
      rightSideComposite.setLayout(gridLayout);


      // Add configuration sections
      for (CustomizeDialogUISection section : sections) {
         if (section instanceof ColumnsCustomUISection) {
            columns_section = (ColumnsCustomUISection) section;
            columns_section.initialize(xViewerToCustomize.getCustomizeMgr(), this);
         }
         addSection(section, rightSideComposite);
      }

      createConfigCustomizationButtonBar(rightSideComposite);

      try {
         loadCustomizeTable();
      }
      catch (Exception ex) {
         XViewerLog.logAndPopup(Activator.class, Level.SEVERE, ex);
      }
      updateButtonEnablements();

      return comp;
   }

   private void recomputeScrolledHeight() {
      int newWidth = rightSideComposite.getSize().x;
      int height = rightSideComposite.computeSize(newWidth, SWT.DEFAULT).y;
      rightSideScroller.setMinHeight(height);
   }

   private void addSection(CustomizeDialogUISection section, final Composite parent) {
      ExpandableComposite expander = new ExpandableComposite(parent, ExpandableComposite.TWISTIE | ExpandableComposite.CLIENT_INDENT | ExpandableComposite.EXPANDED);
      expander.setText(section.getName());
      expander.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
      expander.setExpanded(true);

      final Control columns_section_body = section.createSection(parent);
      columns_section_body.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));

      expander.addExpansionListener(new IExpansionListener() {
         @Override
         public void expansionStateChanging(ExpansionEvent e) {
         }

         @Override
         public void expansionStateChanged(ExpansionEvent e) {
            boolean expanded = e.getState();
            columns_section_body.setVisible(expanded);
            ((GridData) columns_section_body.getLayoutData()).exclude = !expanded;
            recomputeScrolledHeight();
            parent.layout(true);
            rightSideScroller.layout();
         }
      });
   }

   private void createSelectCustomizationSection(Composite comp) {
      // Customization Table and Buttons
      final Composite custComp = new Composite(comp, SWT.NONE);
      final GridData gd_composite_6 = new GridData(SWT.FILL, SWT.FILL, true, true);
      custComp.setLayoutData(gd_composite_6);
      final GridLayout gridLayout_1 = new GridLayout();
      gridLayout_1.marginWidth = 0;
      gridLayout_1.marginHeight = 0;
      custComp.setLayout(gridLayout_1);

      Label selectCustomizationLabel = new Label(custComp, SWT.NONE);
      selectCustomizationLabel.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
      selectCustomizationLabel.setText("Select Customization");

      // Customization Table
      custTable = new XViewerFilteredTree(custComp, SWT.BORDER, new PatternFilter());
      final Tree table_2 = custTable.getViewer().getTree();
      final GridData gd_table_2 = new GridData(SWT.FILL, SWT.FILL, true, true);
      gd_table_2.heightHint = 270;
      gd_table_2.widthHint = 200;
      table_2.setLayoutData(gd_table_2);
      custTable.getViewer().setLabelProvider(new CustomizeDataLabelProvider(xViewerToCustomize));
      custTable.getViewer().setContentProvider(new ArrayTreeContentProvider());
      custTable.getViewer().setSorter(new ViewerSorter() {
         @SuppressWarnings("unchecked")
         @Override
         public int compare(Viewer viewer, Object e1, Object e2) {
            if (((CustomizeData) e1).getName().startsWith("-")) {
               return -1;
            }
            else if (((CustomizeData) e2).getName().startsWith("-")) {
               return 1;
            }
            else {
               return getComparator().compare(((CustomizeData) e1).getName(), ((CustomizeData) e2).getName());
            }
         }
      });
      custTable.getViewer().addSelectionChangedListener(new ISelectionChangedListener() {
         @Override
         public void selectionChanged(SelectionChangedEvent event) {
            handleCustTableSelectionChanged();
            updateButtonEnablements();
            storeCustTableSelection();
         }
      });

      // Customization Table Buttons
      final Composite composite = new Composite(custComp, SWT.NONE);
      composite.setLayoutData(new GridData());
      final GridLayout gridLayout_7 = new GridLayout();
      gridLayout_7.numColumns = 2;
      composite.setLayout(gridLayout_7);

      setDefaultButton = new Button(composite, SWT.NONE);
      setDefaultButton.setLayoutData(new GridData());
      setDefaultButton.setText(SET_AS_DEFAULT);
      setDefaultButton.addSelectionListener(new SelectionAdapter() {
         @Override
         public void widgetSelected(SelectionEvent e) {
            super.widgetSelected(e);
            handleSetDefaultButton();
            updateButtonEnablements();
         }
      });

      deleteButton = new Button(composite, SWT.NONE);
      deleteButton.setLayoutData(new GridData());
      deleteButton.setText("Delete");
      deleteButton.addSelectionListener(new SelectionAdapter() {
         @Override
         public void widgetSelected(SelectionEvent e) {
            super.widgetSelected(e);
            handleDeleteButton();
            updateButtonEnablements();
         }
      });
   }

   private void createConfigCustomizationButtonBar(final Composite composite) {
      // Button block
      final Composite composite_1 = new Composite(composite, SWT.NONE);
      composite_1.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
      final GridLayout gridLayout_10 = new GridLayout();
      gridLayout_10.numColumns = 5;
      composite_1.setLayout(gridLayout_10);

      // Customization Buttons
      saveButton = new Button(composite_1, SWT.NONE);
      saveButton.setText("Save Customization");
      saveButton.addSelectionListener(new SelectionAdapter() {
         @Override
         public void widgetSelected(SelectionEvent e) {
            super.widgetSelected(e);
            handleSaveButton();
         }
      });
   }

   public void columnVisibilityChanged(List<XViewerColumn> visible, List<XViewerColumn> hidden) {
      for (CustomizeDialogUISection section : sections)
         section.columnVisibilityChanged(visible, hidden);
   }

   private void handleSaveButton() {
      try {
         List<CustomizeData> custDatas = new ArrayList<CustomizeData>();
         for (CustomizeData custData : xViewerToCustomize.getCustomizeMgr().getSavedCustDatas()) {
            if (custData.isPersonal()) {
               custDatas.add(custData);
            }
            else if (xViewerToCustomize.getXViewerFactory().isAdmin()) {
               custDatas.add(custData);
            }
         }
         CustomizationDataSelectionDialog diag = new CustomizationDataSelectionDialog(xViewerToCustomize, custDatas);
         if (diag.open() == 0) {
            String name = diag.getEnteredName();
            try {
               CustomizeData diagSelectedCustomizeData = diag.getSelectedCustData();
               String diagEnteredNewName = diag.getEnteredName();
               CustomizeData custData = getConfigCustomizeCustData();
               if (diagEnteredNewName != null) {
                  custData.setName(name);
                  // Set currently selected to newly saved custData
                  selectedCustTableCustData = custData;
               }
               else {
                  custData.setName(diagSelectedCustomizeData.getName());
                  custData.setGuid(diagSelectedCustomizeData.getGuid());
               }
               custData.setPersonal(!diag.isSaveGlobal());
               xViewerToCustomize.getCustomizeMgr().saveCustomization(custData);
            }
            catch (Exception ex) {
               XViewerLog.logAndPopup(Activator.class, Level.SEVERE, ex);
            }
         }
         loadCustomizeTable();
      }
      catch (Exception ex) {
         XViewerLog.logAndPopup(Activator.class, Level.SEVERE, ex);
      }
   }

   /**
    * @return CustomizeData represented by the configuration area
    */
   private CustomizeData getConfigCustomizeCustData() {
      CustomizeData custData = new CustomizeData();
      custData.resetGuid();
      custData.setNameSpace(xViewerToCustomize.getXViewerFactory().getNamespace());

      for (CustomizeDialogUISection section : sections)
         section.saveToCustomData(custData);

      for (XViewerColumn xCol : custData.getColumnData().getColumns())
         xCol.setXViewer(xViewerToCustomize);

      return custData;
   }

   private void handleLoadConfigCustButton() {
      xViewerToCustomize.getCustomizeMgr().loadCustomization(getConfigCustomizeCustData());
      xViewerToCustomize.refresh();
   }

   /**
    * for testing purposes
    */
   protected void handleLoadConfigCust() {
      handleLoadConfigCustButton();
   }

   private void handleSetDefaultButton() {
      try {
         CustomizeData custData = getCustTableSelection();
         if (custData.getName().equals(CustomizeManager.TABLE_DEFAULT_LABEL) || custData.getName().equals(
               CustomizeManager.CURRENT_LABEL)) {
            XViewerLib.popup("ERROR", "Can't set table default or current as default");
            return;
         }
         if (xViewerToCustomize.getCustomizeMgr().isCustomizationUserDefault(custData)) {
            if (MessageDialog.openConfirm(Display.getCurrent().getActiveShell(), "Remove Default",
                  "Remove \"" + custData.getName() + "\" as default for this table?")) {
               xViewerToCustomize.getCustomizeMgr().setUserDefaultCustData(custData, false);
            }
         }
         else if (MessageDialog.openConfirm(Display.getCurrent().getActiveShell(), "Set Default",
               "Set \"" + custData.getName() + "\" as default for this table?")) {
            xViewerToCustomize.getCustomizeMgr().setUserDefaultCustData(custData, true);
         }
         loadCustomizeTable();
      }
      catch (Exception ex) {
         XViewerLog.logAndPopup(Activator.class, Level.SEVERE, ex);
      }
   }

   private void handleDeleteButton() {
      try {
         CustomizeData custSel = getCustTableSelection();
         if (custSel.getName().equals(CustomizeManager.TABLE_DEFAULT_LABEL) || custSel.getName().equals(
               CustomizeManager.CURRENT_LABEL)) {
            XViewerLib.popup("ERROR", "Can't delete defaults.");
            return;
         }
         if (!custSel.isPersonal() && !xViewerToCustomize.getXViewerFactory().isAdmin()) {
            XViewerLib.popup("ERROR", "Global Customizations can only be deleted by admin");
            return;
         }
         if (MessageDialog.openConfirm(Display.getCurrent().getActiveShell(), "Delete Customization",
               "Delete \"" + custSel.getName() + "\" customization?")) {
            xViewerToCustomize.getCustomizeMgr().deleteCustomization(custSel);
            loadCustomizeTable();
            updateButtonEnablements();
         }
      }
      catch (Exception ex) {
         XViewerLog.logAndPopup(Activator.class, Level.SEVERE, ex);
      }
   }

   private void updateButtonEnablements() {
      CustomizeData custData = getCustTableSelection();
      setDefaultButton.setEnabled(xViewerToCustomize.getXViewerFactory().getXViewerCustomizations().isCustomizationPersistAvailable() && custTable.getViewer().getTree().isFocusControl() && custData != null && !custData.getName().equals(
            CustomizeManager.TABLE_DEFAULT_LABEL) && !custData.getName().equals(CustomizeManager.CURRENT_LABEL));
      if (custTable.getViewer().getTree().isFocusControl() && custData != null) {
         try {
            setDefaultButton.setText(xViewerToCustomize.getCustomizeMgr().isCustomizationUserDefault(custData) ? REMOVE_DEFAULT : SET_AS_DEFAULT);
         }
         catch (XViewerException ex) {
            XViewerLog.log(Activator.class, Level.SEVERE, ex);
         }
         setDefaultButton.getParent().layout();
      }
      deleteButton.setEnabled(xViewerToCustomize.getXViewerFactory().getXViewerCustomizations().isCustomizationPersistAvailable() && custTable.getViewer().getTree().isFocusControl() && custData != null);
      saveButton.setEnabled(xViewerToCustomize.getXViewerFactory().getXViewerCustomizations() != null && xViewerToCustomize.getXViewerFactory().getXViewerCustomizations().isCustomizationPersistAvailable());
   }

   private void loadCustomizeTable() throws Exception {
      // Add stored customization data
      List<CustomizeData> custDatas = xViewerToCustomize.getCustomizeMgr().getSavedCustDatas();

      // Add table default customization data
      CustomizeData defaultTableCustData = xViewerToCustomize.getCustomizeMgr().getTableDefaultCustData();
      defaultTableCustData.setName(CustomizeManager.TABLE_DEFAULT_LABEL);
      custDatas.add(defaultTableCustData);

      // Add current customization data generated from actual table
      CustomizeData currentCustData = xViewerToCustomize.getCustomizeMgr().generateCustDataFromTable();
      currentCustData.setName(CustomizeManager.CURRENT_LABEL);
      custDatas.add(currentCustData);

      custTable.getViewer().setInput(custDatas);

      restoreCustTableSelection();

      // If selection not restored, select default
      if (getCustTableSelection() == null) {
         ArrayList<Object> sel = new ArrayList<Object>();
         sel.add(currentCustData);
         custTable.getViewer().setSelection(new StructuredSelection(sel.toArray(new Object[sel.size()])));
         custTable.getViewer().getTree().setFocus();
      }

      updateButtonEnablements();
   }

   private CustomizeData getCustTableSelection() {
      IStructuredSelection selection = (IStructuredSelection) custTable.getViewer().getSelection();
      if (selection.isEmpty()) {
         return null;
      }
      Iterator<?> i = selection.iterator();
      return (CustomizeData) i.next();
   }

   private void handleCustTableSelectionChanged() {
      if (getCustTableSelection() == null) {
         return;
      }
      CustomizeData custData = getCustTableSelection();
      if (custData == null) {
         XViewerLog.logAndPopup(Activator.class, Level.SEVERE, new IllegalStateException("Can't obtain selection Xml"));
         return;
      }

      for (CustomizeDialogUISection section : sections)
         section.readFromCustomData(custData);
      columnVisibilityChanged(columns_section.getVisibleColumns(), columns_section.getHiddenColumns());
   }

   private CustomizeData selectedCustTableCustData = null;

   public void storeCustTableSelection() {
      // Store selected so can re-select after event re-draw
      if (getCustTableSelection() != null) {
         selectedCustTableCustData = getCustTableSelection();
      }
      //      System.out.println("Selection " + selectedCustTableCustData.getName() + " - " + selectedCustTableCustData.getGuid());
   }

   public void restoreCustTableSelection() {
      if (selectedCustTableCustData != null) {
         ArrayList<Object> selected = new ArrayList<Object>();
         selected.add(selectedCustTableCustData);
         custTable.getViewer().setSelection(new StructuredSelection(selected.toArray(new Object[selected.size()])));
         //         System.out.println("Restoring " + selectedCustTableCustData.getName() + " - " + selectedCustTableCustData.getGuid());
      }
   }

   @Override
   protected void buttonPressed(int buttonId) {
      // Ok
      if (buttonId == 0) {
         handleLoadConfigCustButton();
         close();
      }
      // Apply
      else if (buttonId == 1) {
         handleLoadConfigCustButton();
      }
      // Cancel
      else {
         close();
      }
   }

   public String getTitle() {
      return title;
   }

   private ScrolledComposite rightSideScroller;
   private Composite rightSideComposite;
}
